@@ -201,7 +201,9 @@ public class SubsystemResolver {
             try {
                 wiring = resolver.resolve(context);
                 json.put("success", "true");
+                json.put("wiring", toJson(wiring));
             } catch (Exception e) {
+                json.put("success", "false");
                 json.put("exception", e.toString());
                 throw e;
             } finally {
@@ -209,7 +211,7 @@ public class SubsystemResolver {
                         Paths.get(outputFile),
                         StandardCharsets.UTF_8,
                         StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
-                    JsonWriter.write(writer, json);
+                    JsonWriter.write(writer, json, true);
                 }
             }
         } else {
@@ -235,6 +237,37 @@ public class SubsystemResolver {
         return wiring;
     }
 
+    private Object toJson(Map<Resource, List<Wire>> wiring) {
+        Map<String, List<Map<String, Object>>> wires = new HashMap<>();
+        for (Map.Entry<Resource, List<Wire>> reswiring : wiring.entrySet()) {
+            Resource resource = reswiring.getKey();
+            String id = toString(resource);
+            List<Map<String, Object>> reswires = new ArrayList<>();
+            for (Wire w : reswiring.getValue()) {
+                Map<String, Object> rw = new LinkedHashMap<>();
+                rw.put("requirement", toString(w.getRequirement()));
+                rw.put("capability", toString(w.getCapability()));
+                rw.put("requirer", toString(w.getRequirer()));
+                rw.put("provider", toString(w.getProvider()));
+                reswires.add(rw);
+            }
+            wires.put(id, reswires);
+        }
+        return wires;
+    }
+
+    private String toString(Resource r) {
+        return toString(r.getCapabilities(IDENTITY_NAMESPACE).get(0));
+    }
+
+    private String toString(Requirement r) {
+        return BaseClause.toString(null, r.getNamespace(), r.getAttributes(), r.getDirectives());
+    }
+
+    private String toString(Capability c) {
+        return BaseClause.toString(null, c.getNamespace(), c.getAttributes(), c.getDirectives());
+    }
+
     private Object toJson(Repository repository) {
         Requirement req = new RequirementImpl(
                 null,
@@ -277,12 +310,7 @@ public class SubsystemResolver {
     private void addBundleInfos(Subsystem subsystem) {
         if (subsystem != null) {
             String region = getFlatSubsystemsMap().get(subsystem.getName());
-            Map<String, BundleInfo> bis = bundleInfos.get(region);
-            if (bis == null) {
-                bis = new HashMap<>();
-                bundleInfos.put(region, bis);
-            }
-            bis.putAll(subsystem.getBundleInfos());
+            bundleInfos.computeIfAbsent(region, k -> new HashMap<>()).putAll(subsystem.getBundleInfos());
             for (Subsystem child : subsystem.getChildren()) {
                 addBundleInfos(child);
             }
@@ -411,7 +439,7 @@ public class SubsystemResolver {
         for (Map.Entry<Resource, List<Wire>> entry : wiring.entrySet()) {
             final Resource resource = entry.getKey();
             final Requirement requirement = getSubsystemRequirement(resource);
-            if (ResolverUtil.isFragment(resource)) {
+            if (ResolverUtil.isFragment(resource) && requirement != null) {
                 List<Wire> wires = entry.getValue();
                 final Resource host = wires.get(0).getProvider();
                 final Wire wire = findMatchingWire(sf, wiring.get(host));
