@@ -52,9 +52,9 @@ import org.apache.karaf.jpm.impl.ScriptUtils;
 import org.apache.karaf.profile.Profile;
 import org.apache.karaf.profile.ProfileBuilder;
 import org.apache.karaf.profile.ProfileService;
+import org.apache.karaf.shell.support.ansi.SimpleAnsi;
 import org.apache.karaf.util.StreamUtils;
 import org.apache.karaf.util.locks.FileLockUtils;
-import org.fusesource.jansi.Ansi;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.FrameworkUtil;
 import org.osgi.framework.ServiceReference;
@@ -84,7 +84,7 @@ public class InstanceServiceImpl implements InstanceService {
 
     private static final String DEFAULT_SHUTDOWN_COMMAND = "SHUTDOWN";
 
-    public static final String DEFAULT_JAVA_OPTS = "-server -Xmx512M -Dcom.sun.management.jmxremote -XX:+UnlockDiagnosticVMOptions -XX:+UnsyncloadClass";
+    public static final String DEFAULT_JAVA_OPTS = "-server -Xmx512M -Dcom.sun.management.jmxremote -XX:+UnlockDiagnosticVMOptions";
 
     private LinkedHashMap<String, InstanceImpl> proxies = new LinkedHashMap<String, InstanceImpl>();
 
@@ -319,6 +319,7 @@ public class InstanceServiceImpl implements InstanceService {
 
                 mkdir(karafBase, "bin", printOutput);
                 mkdir(karafBase, "etc", printOutput);
+                mkdir(karafBase, "etc/scripts", printOutput);
                 mkdir(karafBase, "system", printOutput);
                 mkdir(karafBase, "deploy", printOutput);
                 mkdir(karafBase, "data", printOutput);
@@ -334,12 +335,33 @@ public class InstanceServiceImpl implements InstanceService {
                     "etc/distribution.info",
                     "etc/equinox-debug.properties",
                     "etc/java.util.logging.properties",
+                    "etc/jmx.acl.cfg",
+                    "etc/jmx.acl.java.lang.Memory.cfg",
+                    "etc/jmx.acl.org.apache.karaf.bundle.cfg",
+                    "etc/jmx.acl.org.apache.karaf.config.cfg",
+                    "etc/jmx.acl.org.apache.karaf.security.jmx.cfg",
+                    "etc/jmx.acl.osgi.compendium.cm.cfg",
                     "etc/jre.properties",
                     "etc/keys.properties",
+                    "etc/org.apache.felix.eventadmin.impl.EventAdmin.cfg",
+                    "etc/org.apache.felix.fileinstall-deploy.cfg",
+                    "etc/org.apache.karaf.command.acl.bundle.cfg",
+                    "etc/org.apache.karaf.command.acl.config.cfg",
+                    "etc/org.apache.karaf.command.acl.feature.cfg",
+                    "etc/org.apache.karaf.command.acl.jaas.cfg",
+                    "etc/org.apache.karaf.command.acl.kar.cfg",
+                    "etc/org.apache.karaf.command.acl.scope_bundle.cfg",
+                    "etc/org.apache.karaf.command.acl.shell.cfg",
+                    "etc/org.apache.karaf.command.acl.system.cfg",
+                    "etc/org.apache.karaf.features.repos.cfg",
+                    "etc/org.apache.karaf.jaas.cfg",
+                    "etc/org.apache.karaf.kar.cfg",
+                    "etc/org.apache.karaf.log.cfg",
                     "etc/org.ops4j.pax.logging.cfg",
                     "etc/org.ops4j.pax.url.mvn.cfg",
                     "etc/shell.init.script",
                     "etc/users.properties",
+                    "etc/scripts/shell.completion.script",
                     FEATURES_CFG
                 };
                 copyResourcesToDir(resources, karafBase, textResources, printOutput);
@@ -691,10 +713,10 @@ public class InstanceServiceImpl implements InstanceService {
                     throw new IllegalStateException("Instance not stopped");
                 }
 
-                println(Ansi.ansi().a("Renaming instance ")
-                        .a(Ansi.Attribute.INTENSITY_BOLD).a(oldName).a(Ansi.Attribute.RESET)
-                        .a(" to ")
-                        .a(Ansi.Attribute.INTENSITY_BOLD).a(newName).a(Ansi.Attribute.RESET).toString());
+                println("Renaming instance "
+                        + SimpleAnsi.INTENSITY_BOLD + oldName + SimpleAnsi.INTENSITY_NORMAL
+                        + " to "
+                        + SimpleAnsi.INTENSITY_BOLD + newName + SimpleAnsi.INTENSITY_NORMAL);
                 // rename directory
                 String oldLocationPath = instance.loc;
                 File oldLocation = new File(oldLocationPath);
@@ -1096,7 +1118,11 @@ public class InstanceServiceImpl implements InstanceService {
                 InputStream is = getResourceStream(resource, resources);
                 OutputStream os = new FileOutputStream(outFile)
             ) {
-                copyStream(is, os);
+                if (is == null) {
+                    logInfo("\tWARNING: unable to find %s", true, resource);
+                } else {
+                    copyStream(is, os);
+                }
             }
         }
     }
@@ -1164,7 +1190,7 @@ public class InstanceServiceImpl implements InstanceService {
     
     private void copyFilteredResourceToDir(String resource, File target, Map<String, URL> resources, Map<String, String> props, boolean printOutput) throws IOException {
         File outFile = new File(target, resource);
-        if( !outFile.exists() ) {
+        if (!outFile.exists()) {
             logDebug("Creating file: %s", printOutput, outFile.getPath());
             try (
                 InputStream is = getResourceStream(resource, resources);
