@@ -22,7 +22,11 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.lang.reflect.Field;
+import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.lang.reflect.Proxy;
 import java.net.URI;
 import java.net.URL;
 import java.net.URLClassLoader;
@@ -30,22 +34,28 @@ import java.security.Provider;
 import java.security.Security;
 import java.util.ArrayList;
 import java.util.List;
-import org.apache.felix.utils.properties.Properties;
 import java.util.StringTokenizer;
+import java.util.concurrent.TimeUnit;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import org.apache.felix.utils.properties.Properties;
 import org.apache.karaf.info.ServerInfo;
+import org.apache.karaf.main.internal.Systemd;
 import org.apache.karaf.main.lock.Lock;
 import org.apache.karaf.main.lock.LockCallBack;
 import org.apache.karaf.main.lock.NoLock;
 import org.apache.karaf.main.util.ArtifactResolver;
 import org.apache.karaf.main.util.BootstrapLogManager;
 import org.apache.karaf.main.util.SimpleMavenResolver;
-import org.apache.karaf.main.util.StringMap;
 import org.apache.karaf.main.util.Utils;
-
-import org.osgi.framework.*;
+import org.apache.karaf.util.config.PropertiesLoader;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleContext;
+import org.osgi.framework.BundleException;
+import org.osgi.framework.Constants;
+import org.osgi.framework.FrameworkEvent;
+import org.osgi.framework.FrameworkListener;
 import org.osgi.framework.launch.Framework;
 import org.osgi.framework.launch.FrameworkFactory;
 import org.osgi.framework.startlevel.BundleStartLevel;
@@ -68,18 +78,19 @@ public class Main {
      */
     public static final String STARTUP_PROPERTIES_FILE_NAME = "startup.properties";
 
+    private static final Logger LOG = Logger.getLogger(Main.class.getName());
 
-    Logger LOG = Logger.getLogger(this.getClass().getName());
+    private final String[] args;
+
+    private final KarafLockCallback lockCallback = new KarafLockCallback();
 
     private ConfigProperties config;
-    private Framework framework = null;
-    private final String[] args;
+    private Framework framework;
     private int exitCode;
     private ShutdownCallback shutdownCallback;
     private KarafActivatorManager activatorManager;
-    private Lock lock;
-    private KarafLockCallback lockCallback;
-    private boolean exiting;
+    private volatile Lock lock;
+    private volatile boolean exiting;
     
     /**
      * <p>
@@ -88,7 +99,7 @@ public class Main {
      * when invoked:
      * </p>
      * <ol>
-     *   <li><i><b>Read the system properties file.<b></i> This is a file
+     *   <li><i><b>Read the system properties file.</b></i> This is a file
      *       containing properties to be pushed into <tt>System.setProperty()</tt>
      *       before starting the framework. This mechanism is mainly shorthand
      *       for people starting the framework from the command line to avoid having
@@ -116,8 +127,7 @@ public class Main {
      *       the desired URL using the <tt>felix.config.properties</tt>
      *       system property; this should be set using the <tt>-D</tt> syntax
      *       when executing the JVM. Refer to the
-     *       <a href="Felix.html#Felix(java.util.Map, java.util.List)">
-     *       <tt>Felix</tt></a> constructor documentation for more
+     *       <tt>Felix</tt> constructor documentation for more
      *       information on the framework configuration options.
      *   </li>
      *   <li><i><b>Perform system property variable substitution on configuration
@@ -151,8 +161,7 @@ public class Main {
      * the configuration property file cannot be found, the framework will appear to
      * be hung or deadlocked. This is not the case, it is executing correctly,
      * there is just no way to interact with it. Refer to the
-     * <a href="Felix.html#Felix(java.util.Map, java.util.List)">
-     * <tt>Felix</tt></a> constructor documentation for more information on
+     * <tt>Felix</tt> constructor documentation for more information on
      * framework configuration options.
      * </p>
      * @param args An array of arguments, all of which are ignored.
@@ -161,6 +170,9 @@ public class Main {
     public static void main(String[] args) throws Exception {
         while (true) {
             boolean restart = false;
+            boolean restartJvm = false;
+            // karaf.restart.jvm take priority over karaf.restart
+            System.setProperty("karaf.restart.jvm", "false");
             System.setProperty("karaf.restart", "false");
             final Main main = new Main(args);
             try {
@@ -177,6 +189,7 @@ public class Main {
                 main.awaitShutdown();
                 boolean stopped = main.destroy();
                 restart = Boolean.getBoolean("karaf.restart");
+                restartJvm = Boolean.getBoolean("karaf.restart.jvm");
                 main.updateInstancePidAfterShutdown();
                 if (!stopped) {
                     if (restart) {
@@ -191,7 +204,9 @@ public class Main {
                 System.err.println("Error occurred shutting down framework: " + ex);
                 ex.printStackTrace();
             } finally {
-                if (!restart) {
+                if (restartJvm && restart) {
+                    System.exit(10);
+                } else if (!restart) {
                     System.exit(main.getExitCode());
                 } else {
                     System.gc();
@@ -216,15 +231,18 @@ public class Main {
     }
 
     public void launch() throws Exception {
-        config = new ConfigProperties();
+        if (config == null) {
+            config = new ConfigProperties();
+        }
+        config.performInit();
         if (config.delayConsoleStart) {
             System.out.println(config.startupMessage);
         }
         String log4jConfigPath = System.getProperty("karaf.etc") + "/org.ops4j.pax.logging.cfg";
         BootstrapLogManager.setProperties(config.props, log4jConfigPath);
-        lockCallback = new KarafLockCallback();
-        InstanceHelper.updateInstancePid(config.karafHome, config.karafBase, true);
-        LOG.addHandler(BootstrapLogManager.getDefaultHandler());
+        /* KARAF-5798: write the PID whether or not the lock has been acquired */
+        InstanceHelper.writePid(config.pidFile);
+        BootstrapLogManager.configureLogger(LOG);
 
         for (String provider : config.securityProviders) {
             addSecurityProvider(provider);
@@ -236,7 +254,10 @@ public class Main {
         // Start up the OSGI framework
         ClassLoader classLoader = createClassLoader(resolver);
         FrameworkFactory factory = loadFrameworkFactory(classLoader);
-        framework = factory.newFramework(new StringMap(config.props, false));
+        framework = factory.newFramework(config.props);
+
+        setLogger();
+
         framework.init();
         framework.getBundleContext().addFrameworkListener(lockCallback);
         framework.start();
@@ -266,10 +287,94 @@ public class Main {
             new StartupListener(LOG, framework.getBundleContext());
         }
         monitor();
+        registerSignalHandler();
+        watchdog();
+    }
+
+    /*
+     * Hack to set felix logger
+     * KARAF-3706: disable the logger related code to avoid the exception
+     * It needs to be revisited when the FELIX-4871 is fixed.
+     */
+    private void setLogger() {
+        try {
+            if (framework.getClass().getName().startsWith("org.apache.felix.")) {
+                Field field = framework.getClass().getDeclaredField("m_logger");
+                field.setAccessible(true);
+                Object logger = field.get(framework);
+                Method method = logger.getClass().getDeclaredMethod("setLogger", Object.class);
+                method.setAccessible(true);
+                method.invoke(logger, new Object() {
+                    public void log(int level, String message, Throwable exception) {
+                        Level lvl;
+                        switch (level) {
+                            case 1:
+                                lvl = Level.SEVERE;
+                                break;
+                            case 2:
+                                lvl = Level.WARNING;
+                                break;
+                            case 3:
+                                lvl = Level.INFO;
+                                break;
+                            case 4:
+                                lvl = Level.FINE;
+                                break;
+                            default:
+                                lvl = Level.FINEST;
+                                break;
+                        }
+                        Logger.getLogger("Felix").log(lvl, message, exception);
+                    }
+                });
+            }
+        } catch (Throwable t) {
+            t.printStackTrace();
+        }
+    }
+
+    private void registerSignalHandler() {
+        if (!Boolean.valueOf(System.getProperty("karaf.handle.sigterm", "true"))) {
+            return;
+        }
+
+        try {
+            final Class<?> signalClass = Class.forName("sun.misc.Signal");
+            final Class<?> signalHandlerClass = Class.forName("sun.misc.SignalHandler");
+
+            Object signalHandler = Proxy.newProxyInstance(getClass().getClassLoader(),
+                new Class<?>[] {
+                    signalHandlerClass
+                },
+                new InvocationHandler() {
+                    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
+                        new Thread() {
+                            @Override
+                            public void run() {
+                                try {
+                                    exiting = true;
+                                    framework.stop();
+                                } catch (BundleException e) {
+                                    e.printStackTrace();
+                                }
+                            }
+                        }.start();
+                        return null;
+                    }
+                }
+            );
+
+            signalClass.getMethod("handle", signalClass, signalHandlerClass).invoke(
+                null,
+                signalClass.getConstructor(String.class).newInstance("TERM"),
+                signalHandler
+            );
+        } catch (Exception e) {
+        }
     }
 
     private void monitor() {
-        new Thread() {
+        new Thread("Karaf Lock Monitor Thread") {
             public void run() {
                 try {
                     doMonitor();
@@ -317,7 +422,37 @@ public class Main {
     Lock getLock() {
         return lock;
     }
-    
+
+    private void watchdog() {
+        if(Boolean.getBoolean("karaf.systemd.enabled")) {
+            new Thread("Karaf Systemd Watchdog Thread") {
+                public void run() {
+                    try {
+                        doWatchdog();
+                    } catch (Exception e) {
+                        e.printStackTrace();
+                    }
+                }
+            }.start();
+        }
+    }
+
+    private void doWatchdog() throws Exception {
+        Systemd systemd = new Systemd();
+        long timeout = systemd.getWatchdogTimeout(TimeUnit.MILLISECONDS);
+
+        int code;
+        while (!exiting && timeout > 0) {
+            code = systemd.notifyWatchdog();
+            if(code < 0) {
+                System.err.println("Systemd sd_notify failed with error code: " + code);
+                break;
+            }
+
+            Thread.sleep(timeout / 2);
+        }
+    }
+
     private ClassLoader createClassLoader(ArtifactResolver resolver) throws Exception {
         List<URL> urls = new ArrayList<URL>();
         urls.add(resolver.resolve(config.frameworkBundle).toURL());
@@ -416,15 +551,17 @@ public class Main {
 
     private List<File> getBundleRepos() {
         List<File> bundleDirs = new ArrayList<File>();
-        File baseSystemRepo = new File(config.karafHome, config.defaultRepo);
-        if (!baseSystemRepo.exists() && baseSystemRepo.isDirectory()) {
-            throw new RuntimeException("system repo folder not found: " + baseSystemRepo.getAbsolutePath());
-        }
-        bundleDirs.add(baseSystemRepo);
-
         File homeSystemRepo = new File(config.karafHome, config.defaultRepo);
+        if (!homeSystemRepo.isDirectory()) {
+            throw new RuntimeException("system repo folder not found: " + homeSystemRepo.getAbsolutePath());
+        }
         bundleDirs.add(homeSystemRepo);
 
+        File baseSystemRepo = new File(config.karafBase, config.defaultRepo);
+        if (baseSystemRepo.isDirectory() && !baseSystemRepo.equals(homeSystemRepo)) {
+            bundleDirs.add(baseSystemRepo);
+        }
+
         String locations = config.bundleLocations;
         if (locations != null) {
             StringTokenizer st = new StringTokenizer(locations, "\" ", true);
@@ -479,6 +616,14 @@ public class Main {
         framework.adapt(FrameworkStartLevel.class).setStartLevel(level);
     }
 
+    public ConfigProperties getConfig() {
+        return config;
+    }
+
+    public void setConfig(ConfigProperties config) {
+        this.config = config;
+    }
+
     public void awaitShutdown() throws Exception {
         if (framework == null) {
             return;
@@ -504,11 +649,13 @@ public class Main {
             return true;
         }
         try {
-            int step = 5000;
-
-            // Notify the callback asap
+            int timeout = config.shutdownTimeout;
+            if (config.shutdownTimeout <= 0) {
+                timeout = Integer.MAX_VALUE;
+            }
+            
             if (shutdownCallback != null) {
-                shutdownCallback.waitingForShutdown(step);
+                shutdownCallback.waitingForShutdown(timeout);
             }
 
             exiting = true;
@@ -525,18 +672,14 @@ public class Main {
                 }.start();
             }
 
-            int timeout = config.shutdownTimeout;
-            if (config.shutdownTimeout <= 0) {
-                timeout = Integer.MAX_VALUE;
-            }
+            int step = 5000;      
             while (timeout > 0) {
                 timeout -= step;
-                if (shutdownCallback != null) {
-                    shutdownCallback.waitingForShutdown(step * 2);
-                }
                 FrameworkEvent event = framework.waitForStop(step);
                 if (event.getType() != FrameworkEvent.WAIT_TIMEDOUT) {
-                    activatorManager.stopKarafActivators();
+                    if (activatorManager != null) {
+                        activatorManager.stopKarafActivators();
+                    }
                     return true;
                 }
             }
@@ -575,6 +718,8 @@ public class Main {
         @Override
         public void lockAquired() {
             LOG.info("Lock acquired. Setting startlevel to " + config.defaultStartLevel);
+            /* KARAF-5798: instance PID should reflect the current running master */
+            InstanceHelper.updateInstancePid(config.karafHome, config.karafBase, true);
             InstanceHelper.setupShutdown(config, framework);
             setStartLevel(config.defaultStartLevel);
         }
